# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V23::Enums::UserListStringRuleItemOperatorEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED     => "UNSPECIFIED",
  UNKNOWN         => "UNKNOWN",
  CONTAINS        => "CONTAINS",
  EQUALS          => "EQUALS",
  STARTS_WITH     => "STARTS_WITH",
  ENDS_WITH       => "ENDS_WITH",
  NOT_EQUALS      => "NOT_EQUALS",
  NOT_CONTAINS    => "NOT_CONTAINS",
  NOT_STARTS_WITH => "NOT_STARTS_WITH",
  NOT_ENDS_WITH   => "NOT_ENDS_WITH"
];

1;
